@@ -22,6 +22,7 @@
 #define BIND_H_
 
 #include "libssh/priv.h"
+#include "libssh/kex.h"
 #include "libssh/session.h"
 
 struct ssh_bind_struct {
@@ -31,7 +32,7 @@ struct ssh_bind_struct {
 
   struct ssh_poll_handle_struct *poll;
   /* options */
-  char *wanted_methods[10];
+  char *wanted_methods[SSH_KEX_METHODS];
   char *banner;
   char *ecdsakey;
   char *dsakey;
