@@ -29,9 +29,9 @@
 
 ssh_string ssh_make_bignum_string(bignum num) {
   ssh_string ptr = NULL;
-  int pad = 0;
-  unsigned int len = bignum_num_bytes(num);
-  unsigned int bits = bignum_num_bits(num);
+  size_t pad = 0;
+  size_t len = bignum_num_bytes(num);
+  size_t bits = bignum_num_bits(num);
 
   if (len == 0) {
       return NULL;
@@ -43,7 +43,9 @@ ssh_string ssh_make_bignum_string(bignum num) {
   }
 
 #ifdef DEBUG_CRYPTO
-  fprintf(stderr, "%d bits, %d bytes, %d padding\n", bits, len, pad);
+  SSH_LOG(SSH_LOG_TRACE,
+          "%zu bits, %zu bytes, %zu padding\n",
+          bits, len, pad);
 #endif /* DEBUG_CRYPTO */
 
   ptr = ssh_string_new(len + pad);
@@ -67,7 +69,8 @@ bignum ssh_make_string_bn(ssh_string string)
     size_t len = ssh_string_len(string);
 
 #ifdef DEBUG_CRYPTO
-    fprintf(stderr, "Importing a %zu bits, %zu bytes object ...\n",
+    SSH_LOG(SSH_LOG_TRACE,
+            "Importing a %zu bits, %zu bytes object ...\n",
             len * 8, len);
 #endif /* DEBUG_CRYPTO */
 
@@ -77,7 +80,7 @@ bignum ssh_make_string_bn(ssh_string string)
 }
 
 /* prints the bignum on stderr */
-void ssh_print_bignum(const char *name, const bignum num)
+void ssh_print_bignum(const char *name, const_bignum num)
 {
     unsigned char *hex = NULL;
     if (num != NULL) {
