@@ -108,11 +108,11 @@ static int pki_private_key_decrypt(ssh_string blob,
 {
     struct ssh_cipher_struct *ciphers = ssh_get_ciphertab();
     struct ssh_cipher_struct cipher;
-    uint8_t key_material[128];
-    char passphrase_buffer[128];
+    uint8_t key_material[128] = {0};
+    char passphrase_buffer[128] = {0};
     size_t key_material_len;
-    ssh_buffer buffer;
-    ssh_string salt;
+    ssh_buffer buffer = NULL;
+    ssh_string salt = NULL;
     uint32_t rounds;
     int cmp;
     int rc;
@@ -159,7 +159,7 @@ static int pki_private_key_decrypt(ssh_string blob,
     if (rc != SSH_ERROR){
         rc = ssh_buffer_unpack(buffer, "Sd", &salt, &rounds);
     }
-    ssh_buffer_free(buffer);
+    SSH_BUFFER_FREE(buffer);
     if (rc == SSH_ERROR){
         return SSH_ERROR;
     }
@@ -233,7 +233,7 @@ ssh_pki_openssh_import(const char *text_key,
                        void *auth_data,
                        bool private)
 {
-    const char *ptr=text_key;
+    const char *ptr = text_key;
     const char *end;
     char *base64;
     int cmp;
@@ -250,7 +250,7 @@ ssh_pki_openssh_import(const char *text_key,
     uint8_t padding;
 
     cmp = strncmp(ptr, OPENSSH_HEADER_BEGIN, strlen(OPENSSH_HEADER_BEGIN));
-    if (cmp != 0){
+    if (cmp != 0) {
         SSH_LOG(SSH_LOG_WARN, "Not an OpenSSH private key (no header)");
         goto out;
     }
@@ -259,15 +259,15 @@ ssh_pki_openssh_import(const char *text_key,
         ptr++;
     }
     end = strstr(ptr, OPENSSH_HEADER_END);
-    if (end == NULL){
+    if (end == NULL) {
         SSH_LOG(SSH_LOG_WARN, "Not an OpenSSH private key (no footer)");
         goto out;
     }
     base64 = malloc(end - ptr + 1);
-    if (base64 == NULL){
+    if (base64 == NULL) {
         goto out;
     }
-    for (i = 0; ptr < end; ptr++){
+    for (i = 0; ptr < end; ptr++) {
         if (!isspace((int)ptr[0])) {
             base64[i] = ptr[0];
             i++;
@@ -276,7 +276,7 @@ ssh_pki_openssh_import(const char *text_key,
     base64[i] = '\0';
     buffer = base64_to_bin(base64);
     SAFE_FREE(base64);
-    if (buffer == NULL){
+    if (buffer == NULL) {
         SSH_LOG(SSH_LOG_WARN, "Not an OpenSSH private key (base64 error)");
         goto out;
     }
@@ -289,21 +289,21 @@ ssh_pki_openssh_import(const char *text_key,
                            &nkeys,
                            &pubkey0,
                            &privkeys);
-    if (rc == SSH_ERROR){
+    if (rc == SSH_ERROR) {
         SSH_LOG(SSH_LOG_WARN, "Not an OpenSSH private key (unpack error)");
         goto out;
     }
     cmp = strncmp(magic, OPENSSH_AUTH_MAGIC, strlen(OPENSSH_AUTH_MAGIC));
-    if (cmp != 0){
+    if (cmp != 0) {
         SSH_LOG(SSH_LOG_WARN, "Not an OpenSSH private key (bad magic)");
         goto out;
     }
     SSH_LOG(SSH_LOG_INFO,
-            "Opening OpenSSH private key: ciphername: %s, kdf: %s, nkeys: %d\n",
+            "Opening OpenSSH private key: ciphername: %s, kdf: %s, nkeys: %d",
             ciphername,
             kdfname,
             nkeys);
-    if (nkeys != 1){
+    if (nkeys != 1) {
         SSH_LOG(SSH_LOG_WARN, "Opening OpenSSH private key: only 1 key supported (%d available)", nkeys);
         goto out;
     }
@@ -327,7 +327,7 @@ ssh_pki_openssh_import(const char *text_key,
                                  kdfoptions,
                                  auth_fn,
                                  auth_data);
-    if (rc == SSH_ERROR){
+    if (rc == SSH_ERROR) {
         goto out;
     }
 
@@ -342,20 +342,20 @@ ssh_pki_openssh_import(const char *text_key,
                         ssh_string_len(privkeys));
 
     rc = ssh_buffer_unpack(privkey_buffer, "dd", &checkint1, &checkint2);
-    if (rc == SSH_ERROR || checkint1 != checkint2){
+    if (rc == SSH_ERROR || checkint1 != checkint2) {
         SSH_LOG(SSH_LOG_WARN, "OpenSSH private key unpack error (correct password?)");
         goto out;
     }
     rc = pki_openssh_import_privkey_blob(privkey_buffer, &key);
-    if (rc == SSH_ERROR){
+    if (rc == SSH_ERROR) {
         goto out;
     }
     comment = ssh_buffer_get_ssh_string(privkey_buffer);
     SAFE_FREE(comment);
     /* verify that the remaining data is correct padding */
-    for (i=1; ssh_buffer_get_len(privkey_buffer) > 0; ++i){
+    for (i = 1; ssh_buffer_get_len(privkey_buffer) > 0; ++i) {
         ssh_buffer_get_u8(privkey_buffer, &padding);
-        if (padding != i){
+        if (padding != i) {
             ssh_key_free(key);
             key = NULL;
             SSH_LOG(SSH_LOG_WARN, "Invalid padding");
@@ -364,11 +364,11 @@ ssh_pki_openssh_import(const char *text_key,
     }
 out:
     if (buffer != NULL) {
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
         buffer = NULL;
     }
     if (privkey_buffer != NULL) {
-        ssh_buffer_free(privkey_buffer);
+        SSH_BUFFER_FREE(privkey_buffer);
         privkey_buffer = NULL;
     }
     SAFE_FREE(magic);
@@ -415,12 +415,13 @@ static int pki_openssh_export_privkey_blob(const ssh_key privkey,
         return SSH_ERROR;
     }
     rc = ssh_buffer_pack(buffer,
-                         "sdPdP",
+                         "sdPdPP",
                          privkey->type_c,
-                         (uint32_t)ED25519_PK_LEN,
-                         (size_t)ED25519_PK_LEN, privkey->ed25519_pubkey,
-                         (uint32_t)ED25519_SK_LEN,
-                         (size_t)ED25519_SK_LEN, privkey->ed25519_privkey);
+                         (uint32_t)ED25519_KEY_LEN,
+                         (size_t)ED25519_KEY_LEN, privkey->ed25519_pubkey,
+                         (uint32_t)(2 * ED25519_KEY_LEN),
+                         (size_t)ED25519_KEY_LEN, privkey->ed25519_privkey,
+                         (size_t)ED25519_KEY_LEN, privkey->ed25519_pubkey);
     return rc;
 }
 
@@ -439,9 +440,9 @@ static int pki_private_key_encrypt(ssh_buffer privkey_buffer,
 {
     struct ssh_cipher_struct *ciphers = ssh_get_ciphertab();
     struct ssh_cipher_struct cipher;
-    uint8_t key_material[128];
+    uint8_t key_material[128] = {0};
     size_t key_material_len;
-    char passphrase_buffer[128];
+    char passphrase_buffer[128] = {0};
     int rc;
     int i;
     int cmp;
@@ -619,26 +620,26 @@ ssh_string ssh_pki_openssh_privkey_export(const ssh_key privkey,
 
         salt = ssh_string_new(16);
         if (salt == NULL){
-            ssh_buffer_free(kdf_buf);
+            SSH_BUFFER_FREE(kdf_buf);
             goto error;
         }
 
         ok = ssh_get_random(ssh_string_data(salt), 16, 0);
         if (!ok) {
-            ssh_buffer_free(kdf_buf);
+            SSH_BUFFER_FREE(kdf_buf);
             goto error;
         }
 
         ssh_buffer_pack(kdf_buf, "Sd", salt, rounds);
         kdf_options = ssh_string_new(ssh_buffer_get_len(kdf_buf));
         if (kdf_options == NULL){
-            ssh_buffer_free(kdf_buf);
+            SSH_BUFFER_FREE(kdf_buf);
             goto error;
         }
         memcpy(ssh_string_data(kdf_options),
                ssh_buffer_get(kdf_buf),
                ssh_buffer_get_len(kdf_buf));
-        ssh_buffer_free(kdf_buf);
+        SSH_BUFFER_FREE(kdf_buf);
         rc = pki_private_key_encrypt(privkey_buffer,
                                      passphrase,
                                      "aes128-cbc",
@@ -700,7 +701,7 @@ ssh_string ssh_pki_openssh_privkey_export(const ssh_key privkey,
     str_len = ssh_buffer_get_len(buffer);
     len = ssh_buffer_get_data(buffer, ssh_string_data(str), str_len);
     if (str_len != len) {
-        ssh_string_free(str);
+        SSH_STRING_FREE(str);
         str = NULL;
     }
 
@@ -708,13 +709,13 @@ error:
     if (privkey_buffer != NULL) {
         void *bufptr = ssh_buffer_get(privkey_buffer);
         explicit_bzero(bufptr, ssh_buffer_get_len(privkey_buffer));
-        ssh_buffer_free(privkey_buffer);
+        SSH_BUFFER_FREE(privkey_buffer);
     }
     SAFE_FREE(pubkey_s);
     SAFE_FREE(kdf_options);
     SAFE_FREE(salt);
     if (buffer != NULL) {
-        ssh_buffer_free(buffer);
+        SSH_BUFFER_FREE(buffer);
     }
 
     return str;
