@@ -30,13 +30,14 @@
 
 #include <sys/types.h>
 #include <pwd.h>
+#include <errno.h>
 
 #define BUFLEN 4096
 static char buffer[BUFLEN];
 
 static int sshd_setup(void **state)
 {
-    torture_setup_sshd_server(state);
+    torture_setup_sshd_server(state, false);
 
     return 0;
 }
@@ -51,12 +52,16 @@ static int session_setup(void **state)
 {
     struct torture_state *s = *state;
     struct passwd *pwd;
+    int rc;
 
     pwd = getpwnam("bob");
     assert_non_null(pwd);
-    setuid(pwd->pw_uid);
 
-    s->ssh.session = torture_ssh_session(TORTURE_SSH_SERVER,
+    rc = setuid(pwd->pw_uid);
+    assert_return_code(rc, errno);
+
+    s->ssh.session = torture_ssh_session(s,
+                                         TORTURE_SSH_SERVER,
                                          NULL,
                                          TORTURE_SSH_USER_ALICE,
                                          NULL);
